@@ -1,7 +1,5 @@
 #!/usr/bin/env perl
 
-# Copyright (C) 2008-2010, Sebastian Riedel.
-
 use strict;
 use warnings;
 
@@ -32,35 +30,12 @@ EOF
 
 # Flash policy server
 $loop->listen(
-    port => 843,
-    cb   => sub {
+    port    => 843,
+    read_cb => sub {
         my ($loop, $id) = @_;
 
-        # Read callback
-        $loop->read_cb(
-            $id => sub {
-                my ($loop, $id, $chunk) = @_;
-
-                # Writing
-                $loop->writing($id);
-            }
-        );
-
-        # Write callback
-        $loop->write_cb(
-            $id => sub {
-                my ($loop, $id) = @_;
-
-                # Finish
-                $loop->drop($id);
-
-                # Write XML
-                return $xml;
-            }
-        );
-
-        # Reading only
-        $loop->not_writing($id);
+        # Write XML
+        $loop->write($id, $xml, sub { shift->drop($id) });
     }
 ) or die "Couldn't create listen socket!\n";
 
