@@ -1,5 +1,3 @@
-# Copyright (C) 2008-2010, Sebastian Riedel.
-
 package Mojo::ByteStream;
 
 use strict;
@@ -24,6 +22,9 @@ use constant PUNYCODE_DAMP         => 700;
 use constant PUNYCODE_INITIAL_BIAS => 72;
 use constant PUNYCODE_INITIAL_N    => 128;
 
+# Core module since Perl 5.9.3
+use constant SHA1 => eval 'use Digest::SHA (); 1';
+
 __PACKAGE__->attr(raw_size => 0);
 
 # Punycode delimiter
@@ -303,23 +304,18 @@ my %UNRESERVED;
     $UNRESERVED{$_}++ for @unreserved;
 }
 
-# Do we have any food that wasn't brutally slaughtered?
-# Well, I think the veal died of loneliness.
 sub import {
-    my ($class, $name) = @_;
-
-    # Shortcut
-    return unless $name;
-
-    # Export
     my $caller = caller;
     no strict 'refs';
-    *{"${caller}::$name"} = sub { Mojo::ByteStream->new(@_) };
+    *{"${caller}::b"} = sub { Mojo::ByteStream->new(@_) }
+      if @_ > 1;
 }
 
+# Do we have any food that wasn't brutally slaughtered?
+# Well, I think the veal died of loneliness.
 sub new {
     my $self = shift->SUPER::new();
-    $self->{bytestream} = defined $_[0] ? $_[0] : '';
+    $self->{bytestream} = @_ < 2 ? defined $_[0] ? $_[0] : '' : join('', @_);
     $self->{raw_size} = length $self->{bytestream};
     return $self;
 }
@@ -348,21 +344,30 @@ sub b64_decode {
 sub b64_encode {
     my $self = shift;
     utf8::encode $self->{bytestream} if utf8::is_utf8 $self->{bytestream};
-    $self->{bytestream} = MIME::Base64::encode_base64($self->{bytestream});
+    $self->{bytestream} =
+      MIME::Base64::encode_base64($self->{bytestream}, shift);
     return $self;
 }
 
 sub camelize {
     my $self = shift;
 
-    # Split
-    my @words = split /_/, $self->{bytestream};
+    my @parts;
+    for my $part (split /-/, $self->{bytestream}) {
 
-    # Case
-    @words = map {ucfirst} map {lc} @words;
+        # Junk
+        next unless $part;
 
-    # Join
-    $self->{bytestream} = join '', @words;
+        # Split
+        my @words = split /_/, $part;
+
+        # Case
+        @words = map {ucfirst} map {lc} @words;
+
+        # Camelize
+        push @parts, join '', @words;
+    }
+    $self->{bytestream} = join '::', @parts;
 
     return $self;
 }
@@ -380,17 +385,22 @@ sub decamelize {
     my $self = shift;
 
     # Shortcut
-    return $self if $self->{bytestream} !~ /^[A-Z]+/;
+    return $self if $self->{bytestream} !~ /^[A-Z\:]+/;
 
-    # Split
-    my @words;
-    push @words, $1 while ($self->{bytestream} =~ s/([A-Z]{1}[^A-Z]*)//);
+    my @parts;
+    for my $part (split /\:\:/, $self->{bytestream}) {
 
-    # Case
-    @words = map {lc} @words;
+        # Split
+        my @words;
+        push @words, $1 while ($part =~ s/([A-Z]{1}[^A-Z]*)//);
 
-    # Join
-    $self->{bytestream} = join '_', @words;
+        # Case
+        @words = map {lc} @words;
+
+        # Join
+        push @parts, join '_', @words;
+    }
+    $self->{bytestream} = join '-', @parts;
 
     return $self;
 }
@@ -454,21 +464,9 @@ sub get_line {
     return $line;
 }
 
-sub hmac_md5_sum {
-    my ($self, $secret) = @_;
+sub hmac_md5_sum { shift->_hmac(\&_md5, @_) }
 
-    #Secret
-    $secret ||= 'Very unsecure!';
-    $secret = _md5_sum($secret) if length $secret > 64;
-
-    # HMAC
-    my $ipad = $secret ^ (chr(0x36) x 64);
-    my $opad = $secret ^ (chr(0x5c) x 64);
-    $self->{bytestream} =
-      _md5_sum($opad . _md5_sum($ipad . $self->{bytestream}));
-
-    return $self;
-}
+sub hmac_sha1_sum { shift->_hmac(\&_sha1, @_) }
 
 sub html_escape {
     my $self = shift;
@@ -505,8 +503,16 @@ sub html_unescape {
     return $self;
 }
 
+sub md5_bytes {
+    my $self = shift;
+    utf8::encode $self->{bytestream} if utf8::is_utf8 $self->{bytestream};
+    $self->{bytestream} = _md5($self->{bytestream});
+    return $self;
+}
+
 sub md5_sum {
     my $self = shift;
+    utf8::encode $self->{bytestream} if utf8::is_utf8 $self->{bytestream};
     $self->{bytestream} = Digest::MD5::md5_hex($self->{bytestream});
     return $self;
 }
@@ -695,10 +701,42 @@ sub remove {
     return substr $self->{bytestream}, 0, $length, $chunk;
 }
 
+sub say {
+    my ($self, $handle) = @_;
+    $handle ||= \*STDOUT;
+    utf8::encode $self->{bytestream} if utf8::is_utf8 $self->{bytestream};
+    print $handle $self->{bytestream}, "\n";
+}
+
+sub sha1_bytes {
+    my $self = shift;
+    utf8::encode $self->{bytestream} if utf8::is_utf8 $self->{bytestream};
+    $self->{bytestream} = _sha1($self->{bytestream});
+    return $self;
+}
+
+sub sha1_sum {
+    my $self = shift;
+    die <<'EOF' unless SHA1;
+Module "Digest::SHA" not present in this version of Perl.
+Please install it manually or upgrade Perl to at least version 5.10.
+EOF
+    utf8::encode $self->{bytestream} if utf8::is_utf8 $self->{bytestream};
+    $self->{bytestream} = Digest::SHA::sha1_hex($self->{bytestream});
+    return $self;
+}
+
 sub size { length shift->{bytestream} }
 
 sub to_string { shift->{bytestream} }
 
+sub trim {
+    my $self = shift;
+    $self->{bytestream} =~ s/^\s*//;
+    $self->{bytestream} =~ s/\s*$//;
+    return $self;
+}
+
 sub unquote {
     my $self = shift;
 
@@ -779,8 +817,24 @@ sub _adapt {
         / ($delta + PUNYCODE_SKEW));
 }
 
-# Helper for hmac_md5_sum
-sub _md5_sum { Mojo::ByteStream->new(shift)->md5_sum->to_string }
+sub _hmac {
+    my ($self, $cb, $secret) = @_;
+
+    #Secret
+    $secret ||= 'Very unsecure!';
+    $secret = $cb->($secret) if length $secret > 64;
+
+    # HMAC
+    my $ipad = $secret ^ (chr(0x36) x 64);
+    my $opad = $secret ^ (chr(0x5c) x 64);
+    $self->{bytestream} = unpack 'H*',
+      $cb->($opad . $cb->($ipad . $self->{bytestream}));
+
+    return $self;
+}
+
+# Helper for md5_bytes
+sub _md5 { Digest::MD5::md5(shift) }
 
 # Helper for url_sanitize
 sub _sanitize {
@@ -792,6 +846,15 @@ sub _sanitize {
     return '%' . uc $hex;
 }
 
+# Helper for sha1_bytes
+sub _sha1 {
+    die <<'EOF' unless SHA1;
+Module "Digest::SHA" not present in this version of Perl.
+Please install it manually or upgrade Perl to at least version 5.10.
+EOF
+    Digest::SHA::sha1(shift);
+}
+
 # Helper for html_unescape
 sub _unescape {
     my ($num, $entitie, $hex) = @_;
@@ -829,12 +892,17 @@ Mojo::ByteStream - ByteStream
     $stream->encode('UTF-8');
     $stream->decode('UTF-8');
     $stream->hmac_md5_sum('secret');
+    $stream->hmac_sha1_sum('secret');
     $stream->html_escape;
     $stream->html_unescape;
+    $stream->md5_bytes;
     $stream->md5_sum;
     $stream->qp_encode;
     $stream->qp_decode;
     $stream->quote;
+    $stream->sha1_bytes;
+    $stream->sha1_sum;
+    $stream->trim;
     $stream->unquote;
     $stream->url_escape;
     $stream->url_sanitize;
@@ -847,15 +915,15 @@ Mojo::ByteStream - ByteStream
 
     my $stream2 = $stream->clone;
     print $stream2->to_string;
+    $stream2->say;
 
     # Chained
     my $stream = Mojo::ByteStream->new('foo bar baz')->quote;
     $stream = $stream->unquote->encode('UTF-8)->b64_encode;
     print "$stream";
 
-    # Constructor alias
+    # Alternative constructor
     use Mojo::ByteStream 'b';
-
     my $stream = b('foobarbaz')->html_escape;
 
     # Buffering
@@ -906,6 +974,7 @@ Base 64 decode bytestream.
 =head2 C<b64_encode>
 
     $stream = $stream->b64_encode;
+    $stream = $stream->b64_encode('');
 
 Base 64 encode bytestream.
 
@@ -972,6 +1041,13 @@ Lines are expected to end with C<0x0d 0x0a> or C<0x0a>.
 
 Turn bytestream into HMAC-MD5 checksum of old content.
 
+=head2 C<hmac_sha1_sum>
+
+    $stream = $stream->hmac_sha1_sum($secret);
+
+Turn bytestream into HMAC-SHA1 checksum of old content.
+Note that Perl 5.10 or L<Digest::SHA> are required for C<SHA1> support.
+
 =head2 C<html_escape>
 
     $stream = $stream->html_escape;
@@ -984,6 +1060,12 @@ HTML escape bytestream.
 
 HTML unescape bytestream.
 
+=head2 C<md5_bytes>
+
+    $stream = $stream->md5_bytes;
+
+Turn bytestream into binary MD5 checksum of old content.
+
 =head2 C<md5_sum>
 
     $stream = $stream->md5_sum;
@@ -1027,6 +1109,27 @@ Quote bytestream.
 
 Remove a specific number of bytes from bytestream.
 
+=head2 C<say>
+
+    $stream->say;
+    $stream->say(*STDERR);
+
+Print bytestream to handle or STDOUT and append a newline.
+
+=head2 C<sha1_bytes>
+
+    $stream = $stream->sha1_bytes;
+
+Turn bytestream into binary SHA1 checksum of old content.
+Note that Perl 5.10 or L<Digest::SHA> are required for C<SHA1> support.
+
+=head2 C<sha1_sum>
+
+    $stream = $stream->sha1_sum;
+
+Turn bytestream into SHA1 checksum of old content.
+Note that Perl 5.10 or L<Digest::SHA> are required for C<SHA1> support.
+
 =head2 C<size>
 
     my $size = $stream->size;
@@ -1039,6 +1142,12 @@ Size of bytestream.
 
 Stringify bytestream.
 
+=head2 C<trim>
+
+    $stream = $stream->trim;
+
+Trim whitespace characters from both ends of bytestream.
+
 =head2 C<unquote>
 
     $stream = $stream->unquote;
