@@ -1,5 +1,3 @@
-# Copyright (C) 2008-2010, Sebastian Riedel.
-
 package Mojo::Command::Cgi;
 
 use strict;
@@ -19,7 +17,6 @@ usage: $0 cgi [OPTIONS]
 
 These options are available:
   --nph      Enable non-parsed-header mode.
-  --reload   Automatically reload application when the source code changes.
 EOF
 
 # Hi, Super Nintendo Chalmers!
@@ -28,11 +25,8 @@ sub run {
     my $cgi  = Mojo::Server::CGI->new;
 
     # Options
-    @ARGV = @_ if @_;
-    GetOptions(
-        nph    => sub { $cgi->nph(1) },
-        reload => sub { $cgi->reload(1) }
-    );
+    local @ARGV = @_ if @_;
+    GetOptions(nph => sub { $cgi->nph(1) });
 
     # Run
     $cgi->run;
