@@ -1,5 +1,3 @@
-# Copyright (C) 2008-2010, Sebastian Riedel.
-
 package Mojo::Commands;
 
 use strict;
@@ -30,8 +28,7 @@ sub run {
     my ($self, $name, @args) = @_;
 
     # Try to detect environment
-    my $detect = $self->_detect;
-    $name = $detect if $detect && !$ENV{MOJO_NO_DETECT};
+    $name = $self->_detect($name) unless $ENV{MOJO_NO_DETECT};
 
     # Run command
     if ($name && $name =~ /^\w+$/ && ($name ne 'help' || $args[0])) {
@@ -74,24 +71,25 @@ sub run {
         return $help ? $command->help : $command->run(@args);
     }
 
-    # Try all namspaces
+    # Try all namespaces
     my $commands = [];
     my $seen     = {};
     for my $namespace (@{$self->namespaces}) {
 
         # Search
-        my $found = Mojo::Loader->search($namespace);
+        if (my $modules = Mojo::Loader->search($namespace)) {
+            for my $module (@$modules) {
 
-        for my $module (@$found) {
+                # Load
+                if (my $e = Mojo::Loader->load($module)) { die $e }
 
-            # Load
-            if (my $e = Mojo::Loader->load($module)) { die $e }
-
-            # Seen
-            my $command = $module;
-            $command =~ s/^$namespace\:://;
-            push @$commands, [$command => $module] unless $seen->{$command};
-            $seen->{$command} = 1;
+                # Seen
+                my $command = $module;
+                $command =~ s/^$namespace\:://;
+                push @$commands, [$command => $module]
+                  unless $seen->{$command};
+                $seen->{$command} = 1;
+            }
         }
     }
 
@@ -142,13 +140,17 @@ sub start {
 }
 
 sub _detect {
-    my $self = shift;
+    my ($self, $name) = @_;
 
     # PSGI (Plack only for now)
     return 'psgi' if defined $ENV{PLACK_ENV};
 
+    # No further detection if we have a name
+    return $name if $name;
+
     # CGI
-    return 'cgi' if defined $ENV{PATH_INFO};
+    return 'cgi'
+      if defined $ENV{PATH_INFO} || defined $ENV{GATEWAY_INTERFACE};
 
     # FastCGI
     return 'fastcgi' unless defined $ENV{PATH};
