@@ -1,5 +1,3 @@
-# Copyright (C) 2008-2010, Sebastian Riedel.
-
 package Mojo::Content::MultiPart;
 
 use strict;
@@ -39,7 +37,7 @@ sub body_size {
     # Calculate length of whole body
     my $boundary_length = length($boundary) + 6;
     my $length          = 0;
-    $length += $boundary_length;
+    $length += $boundary_length - 2;
     for my $part (@{$self->parts}) {
 
         # Header
@@ -61,8 +59,8 @@ sub build_boundary {
     # Check for existing boundary
     my $headers = $self->headers;
     my $type = $headers->content_type || '';
-    $type =~ /boundary=\"?([^\s\"]+)\"?/i;
-    my $boundary = $1;
+    my $boundary;
+    $type =~ /boundary=\"?([^\s\"]+)\"?/i and $boundary = $1;
     return $boundary if $boundary;
 
     # Generate and check boundary
@@ -97,11 +95,10 @@ sub get_body_chunk {
     # Multipart
     my $boundary        = $self->build_boundary;
     my $boundary_length = length($boundary) + 6;
-    my $length          = $boundary_length;
+    my $length          = $boundary_length - 2;
 
     # First boundary
-    return substr "\x0d\x0a--$boundary\x0d\x0a", $offset
-      if $length > $offset;
+    return substr "--$boundary\x0d\x0a", $offset if $length > $offset;
 
     # Parts
     my $parts = $self->parts;
@@ -144,7 +141,8 @@ sub parse {
     return $self if $self->body_cb;
 
     # Upgrade state
-    $self->state('multipart_preamble') if $self->is_state('body');
+    $self->{_state} = 'multipart_preamble'
+      if ($self->{_state} || '') eq 'body';
 
     # Parse multipart content
     $self->_parse_multipart;
@@ -161,26 +159,26 @@ sub _parse_multipart {
     my $boundary = $1;
 
     # Boundary missing
-    return $self->error(400) unless $boundary;
+    return $self->error('Multipart boundary missing.', 400) unless $boundary;
 
     # Parse
     while (1) {
 
         # Done
-        last if $self->is_state('done', 'error');
+        last if $self->is_done;
 
         # Preamble
-        if ($self->is_state('multipart_preamble')) {
+        if (($self->{_state} || '') eq 'multipart_preamble') {
             last unless $self->_parse_multipart_preamble($boundary);
         }
 
         # Boundary
-        elsif ($self->is_state('multipart_boundary')) {
+        elsif (($self->{_state} || '') eq 'multipart_boundary') {
             last unless $self->_parse_multipart_boundary($boundary);
         }
 
         # Body
-        elsif ($self->is_state('multipart_body')) {
+        elsif (($self->{_state} || '') eq 'multipart_body') {
             last unless $self->_parse_multipart_body($boundary);
         }
     }
@@ -205,7 +203,7 @@ sub _parse_multipart_body {
     # Store chunk
     my $chunk = $buffer->remove($pos);
     $self->parts->[-1] = $self->parts->[-1]->parse($chunk);
-    $self->state('multipart_boundary');
+    $self->{_state} = 'multipart_boundary';
     return 1;
 }
 
@@ -219,7 +217,7 @@ sub _parse_multipart_boundary {
 
         # New part
         push @{$self->parts}, Mojo::Content::Single->new(relaxed => 1);
-        $self->state('multipart_body');
+        $self->{_state} = 'multipart_body';
         return 1;
     }
 
@@ -229,7 +227,7 @@ sub _parse_multipart_boundary {
         $buffer->remove(length $end);
 
         # Done
-        $self->done;
+        $self->{_state} = 'done';
     }
 
     return;
@@ -245,7 +243,7 @@ sub _parse_multipart_preamble {
         $buffer->remove($pos, "\x0d\x0a");
 
         # Parse boundary
-        $self->state('multipart_boundary');
+        $self->{_state} = 'multipart_boundary';
         return 1;
     }
 
