@@ -1,5 +1,3 @@
-# Copyright (C) 2008-2010, Sebastian Riedel.
-
 package Mojo::Cookie::Request;
 
 use strict;
@@ -96,7 +94,8 @@ Mojo::Cookie::Request - HTTP 1.1 Request Cookie Container
 
 =head1 DESCRIPTION
 
-L<Mojo::Cookie::Request> is a container for HTTP 1.1 request cookies.
+L<Mojo::Cookie::Request> is a container for HTTP 1.1 request cookies as
+described in RFC 2965.
 
 =head1 ATTRIBUTES
 
