@@ -1,5 +1,3 @@
-# Copyright (C) 2008-2010, Sebastian Riedel.
-
 package Mojo::Server::PSGI;
 
 use strict;
@@ -7,7 +5,7 @@ use warnings;
 
 use base 'Mojo::Server';
 
-use constant CHUNK_SIZE => $ENV{MOJO_CHUNK_SIZE} || 8192;
+use constant CHUNK_SIZE => $ENV{MOJO_CHUNK_SIZE} || 262144;
 
 # Things aren't as happy as they used to be down here at the unemployment
 # office.
@@ -27,7 +25,7 @@ sub run {
     $tx->local_port($env->{SERVER_PORT});
 
     # Request body
-    while (!$req->is_finished) {
+    while (!$req->is_done) {
         my $read = $env->{'psgi.input'}->read(my $buffer, CHUNK_SIZE, 0);
         last if $read == 0;
         $req->parse($buffer);
@@ -51,20 +49,23 @@ sub run {
     }
 
     # Response body
-    my $body = Mojo::Server::PSGI::Handle->new(res => $res);
+    my $body = Mojo::Server::PSGI::_Handle->new(_res => $res);
+
+    # Finish transaction
+    $tx->finished->($tx);
 
     return [$status, \@headers, $body];
 }
 
-package Mojo::Server::PSGI::Handle;
+package Mojo::Server::PSGI::_Handle;
 
 use strict;
 use warnings;
 
 use base 'Mojo::Base';
 
-__PACKAGE__->attr(offset => 0);
-__PACKAGE__->attr('res');
+__PACKAGE__->attr(_offset => 0);
+__PACKAGE__->attr('_res');
 
 sub close { }
 
@@ -72,9 +73,9 @@ sub getline {
     my $self = shift;
 
     # Blocking read
-    my $offset = $self->offset;
+    my $offset = $self->_offset;
     while (1) {
-        my $chunk = $self->res->get_body_chunk($offset);
+        my $chunk = $self->_res->get_body_chunk($offset);
 
         # No content yet, try again
         unless (defined $chunk) {
@@ -87,7 +88,7 @@ sub getline {
 
         # Content
         $offset += length $chunk;
-        $self->offset($offset);
+        $self->_offset($offset);
         return $chunk;
     }
 
