@@ -1,5 +1,3 @@
-# Copyright (C) 2008-2010, Sebastian Riedel.
-
 package Mojo::Template;
 
 use strict;
@@ -13,7 +11,7 @@ use IO::File;
 use Mojo::ByteStream;
 use Mojo::Exception;
 
-use constant CHUNK_SIZE => $ENV{MOJO_CHUNK_SIZE} || 8192;
+use constant CHUNK_SIZE => $ENV{MOJO_CHUNK_SIZE} || 262144;
 
 __PACKAGE__->attr([qw/auto_escape compiled namespace/]);
 __PACKAGE__->attr([qw/append code prepend/] => '');
@@ -115,7 +113,7 @@ sub build {
             if ($type eq 'cpst') {
 
                 # Start block
-                $cpst = " sub { shift; my \$_M = ''; ";
+                $cpst = " sub { my \$_M = ''; ";
             }
         }
     }
@@ -126,9 +124,9 @@ sub build {
     my $namespace = $self->namespace || ref $self;
     $lines[0] ||= '';
     $lines[0] =
-      qq/package $namespace; sub { my \$_M = ''; $HELPERS; $prepend;/
+      "package $namespace; sub { my \$_M = ''; $HELPERS; $prepend; do {"
       . $lines[0];
-    $lines[-1] .= qq/$append; return \$_M; };/;
+    $lines[-1] .= qq/$append; \$_M; } };/;
 
     $self->code(join "\n", @lines);
     return $self;
@@ -214,15 +212,33 @@ sub parse {
         |
         $tag_start                           # Code
         |
+        $trim$capture_start$tag_end          # Trim end (start)
+        |
         $trim$tag_end                        # Trim end
         |
+        $capture_start$tag_end               # End (start)
+        |
         $tag_end                             # End
         )
     /x;
 
+    # Capture regex
     my $token_capture_re =
       qr/^($tag_start|$tag_end)($capture_end|$capture_start)/;
 
+    # Tag end regex
+    my $end_re = qr/
+        ^(
+        $trim$capture_start$tag_end   # Trim end (start)
+        )|(
+        $capture_start$tag_end        # End (start)
+        )|(
+        $trim$tag_end                 # Trim end
+        )|
+        $tag_end                      # End
+        $
+    /x;
+
     # Tokenize
     my $state                = 'text';
     my $multiline_expression = 0;
@@ -303,10 +319,13 @@ sub parse {
             }
 
             # End
-            if ($state ne 'text' && $token =~ /^($trim$tag_end)|$tag_end$/) {
+            if ($state ne 'text' && $token =~ /$end_re/) {
+
+                # Capture start
+                splice @token, -2, 0, 'cpst', undef if $1 || $2;
 
                 # Trim previous text
-                if ($1) {
+                if ($1 || $3) {
                     $trimming = 1;
 
                     # Trim current line
@@ -556,7 +575,7 @@ Whitespace characters around tags can be trimmed with a special tag ending.
 
 You can capture whole template blocks for reuse later.
 
-    <%{ my $block = %>
+    <% my $block = {%>
         <% my $name = shift; =%>
         Hello <%= $name %>.
     <%}%>
@@ -669,7 +688,7 @@ Character indicating the end of a capture block, defaults to C<}>.
 
 Character indicating the start of a capture block, defaults to C<{>.
 
-    <%{ my $block = %>
+    <% my $block = {%>
         Some data!
     <%}%>
 
