@@ -1,5 +1,3 @@
-# Copyright (C) 2008-2010, Sebastian Riedel.
-
 package Mojolicious::Plugin::AgentCondition;
 
 use strict;
@@ -15,13 +13,13 @@ sub register {
     # Agent
     $app->routes->add_condition(
         agent => sub {
-            my ($r, $tx, $captures, $pattern) = @_;
+            my ($r, $c, $captures, $pattern) = @_;
 
             # Pattern
             return unless $pattern && ref $pattern eq 'Regexp';
 
             # Match
-            my $agent = $tx->req->headers->user_agent;
+            my $agent = $c->req->headers->user_agent;
             return $captures if $agent && $agent =~ $pattern;
 
             # Nothing
